module V2Intuity
  module Endpoints
    class Endpoint
      attr_accessor :base_uri, :org_id, :id, :endpoint, :path, :token

      ENDPOINTS = { measurements: 'measurements', sleeps: 'sleeps',
                    nutrition: 'nutrition', summaries: 'summaries',
                    workouts: 'workouts', users: 'users',
                    marketplace: 'marketplace', streams: 'streams' }.freeze

      def initialize(options = {})
        @endpoint = options[:endpoint]
        @path = options.fetch(:path, 'organizations')
        @id = options[:id]
        add_constants(options)
      end

      def uri(options = {})
        uri_options(options)
        if @id.nil?
          "#{@base_uri}/#{@path}/#{@org_id}/#{@endpoint}"
        else
          "#{@base_uri}/#{@path}/#{@org_id}/users/#{@id}/#{@endpoint}"
        end
      end

      def uri_options(options = {})
        @base_uri ||= options[:base_uri]
        @org_id ||= options[:org_id]
        @id ||= options[:id]

        raise 'org_id is needed' unless @org_id
        raise 'base_uri is needed' unless @base_uri
      end

      def sign_with
        [:access_token]
      end

      private

      def add_constants(options = {})
        @base_uri  = options[:base_uri]
        @token     = options[:token]
        @org_id    = options[:org_id]
      end
    end
  end
end
