module V2Intuity
  module Users
    class User
      attr_accessor :uid, :country_code, :time_zone,
                    :validic_token, :applications, :devices

      def initialize(options = {})
        @validic_token = options.fetch(:validic_token, nil)
        @uid = options.fetch(:uid, nil)
        @time_zone = options.fetch(:location, nil)
        @country_code = options.fetch(:country_code, nil)
      end

      def send_request(method, client)
        endpoint, options = build_options(method)
        client.build_and_send_request(endpoint, options)
      end

      # method refreshes marketplace user token.
      def refresh_marketplace(client)
        raise 'uid is required' unless uid

        options = { method: V2Intuity::Request::METHODS[:post], id: uid }
        endpoint = V2Intuity::Endpoints::Marketplace.new

        client.build_and_send_request(endpoint, options)
      end

      # method finds the marketplace connected and disconnected json response
      def get_marketplace(client, format_redirect = 'json', redirect_uri = V2Intuity::Endpoints::Marketplace::REDIRECT_URI) # marketplace user defined token
        raise 'validic token is required' unless validic_token

        endpoint, options = build_options_marketplace(validic_token, format_redirect, redirect_uri)

        client.build_and_send_request(endpoint, options)
      end

      private

      def build_options_marketplace(token, format_redirect, redirect_uri) # marketplace user defined token
        query_filters = []
        query_filters << V2Intuity::QueryFilters::AccessToken.new(value: token)
        query_filters << V2Intuity::QueryFilters::FormatRedirect.new(value: format_redirect)

        options = { query_filters: query_filters, method: V2Intuity::Request::METHODS[:get] }
        endpoint = V2Intuity::Endpoints::Marketplace.new
        endpoint.define_redirect_uri(redirect_uri) ## overrides client's base_uri

        [endpoint, options]
      end


      def build_options(method, query_filters = [])
        raise 'uid is required' unless uid

        case method
        when V2Intuity::Request::METHODS[:delete], V2Intuity::Request::METHODS[:get]
          options = { method: method, id: uid }
        when V2Intuity::Request::METHODS[:post]
          body = { uid: uid, location: { timezone: time_zone, country_code: country_code }.compact }
          options = { method: method, body: body.compact }
        when V2Intuity::Request::METHODS[:put]
          body = { location: { timezone: time_zone, country_code: country_code }.compact }
          options = { method: method, body: body.compact, id: uid }
        else
          raise 'incorrect method verb'
        end
        options[:query_filters] = query_filters unless query_filters.to_s.nil?

        [V2Intuity::Endpoints::User.new, options] 
      end
    end
  end
end
